#ifndef STENCIL_DESCRIPTOR_HPP
#define STENCIL_DESCRIPTOR_HPP

#include <utils/PugsMacros.hpp>

#include <cstddef>

class StencilDescriptor
{
 public:
  enum class ConnectionType : int
  {
    by_nodes = 1,
    by_edges = 2,
    by_faces = 3,
    by_cells = 4
  };

 private:
  size_t m_number_of_layers;
  ConnectionType m_connection_type;

 public:
  PUGS_INLINE
  const size_t&
  numberOfLayers() const
  {
    return m_number_of_layers;
  };

  PUGS_INLINE
  const ConnectionType&
  connectionType() const
  {
    return m_connection_type;
  };

  PUGS_INLINE
  friend bool
  operator==(const StencilDescriptor& sd0, const StencilDescriptor& sd1)
  {
    return (sd0.m_number_of_layers == sd1.m_number_of_layers) and (sd0.m_connection_type == sd1.m_connection_type);
  }

  PUGS_INLINE
  friend bool
  operator!=(const StencilDescriptor& sd0, const StencilDescriptor& sd1)
  {
    return not(sd0 == sd1);
  }

  StencilDescriptor(const size_t number_of_layers, const ConnectionType type)
    : m_number_of_layers{number_of_layers}, m_connection_type{type}
  {}

  StencilDescriptor(const StencilDescriptor&) = default;
  StencilDescriptor(StencilDescriptor&&)      = default;

  ~StencilDescriptor() = default;
};

#endif   // STENCIL_DESCRIPTOR_HPP
